local __exports = LibStub:NewLibrary("ovale/states/Warlock", 90108)
if not __exports then return end
local __class = LibStub:GetLibrary("tslib").newClass
local __imports = {}
__imports.__enginecondition = LibStub:GetLibrary("ovale/engine/condition")
__imports.returnConstant = __imports.__enginecondition.returnConstant
local tonumber = tonumber
local pairs = pairs
local GetTime = GetTime
local find = string.find
local pow = math.pow
local returnConstant = __imports.returnConstant
local customAuras = {
    [80240] = {
        customId = -80240,
        duration = 10,
        stacks = 1,
        auraName = "active_havoc"
    }
}
local demonData = {
    [55659] = {
        duration = 15
    },
    [98035] = {
        duration = 12
    },
    [103673] = {
        duration = 12
    },
    [11859] = {
        duration = 25
    },
    [89] = {
        duration = 25
    },
    [143622] = {
        duration = 12
    },
    [135002] = {
        duration = 15
    },
    [17252] = {
        duration = 15
    },
    [135816] = {
        duration = 15
    }
}
__exports.OvaleWarlockClass = __class(nil, {
    constructor = function(self, ovale, ovaleAura, ovalePaperDoll, ovaleSpellBook, future, power, combatLogEvent)
        self.ovale = ovale
        self.ovaleAura = ovaleAura
        self.ovalePaperDoll = ovalePaperDoll
        self.ovaleSpellBook = ovaleSpellBook
        self.future = future
        self.power = power
        self.combatLogEvent = combatLogEvent
        self.demonsCount = {}
        self.serial = 1
        self.handleInitialize = function()
            if self.ovale.playerClass == "WARLOCK" then
                self.combatLogEvent.registerEvent("SPELL_SUMMON", self, self.handleCombatLogEvent)
                self.combatLogEvent.registerEvent("SPELL_CAST_SUCCESS", self, self.handleCombatLogEvent)
                self.demonsCount = {}
            end
        end
        self.handleDisable = function()
            if self.ovale.playerClass == "WARLOCK" then
                self.combatLogEvent.unregisterAllEvents(self)
            end
        end
        self.handleCombatLogEvent = function(cleuEvent)
            local cleu = self.combatLogEvent
            if cleu.sourceGUID == self.ovale.playerGUID then
                self.serial = self.serial + 1
                if cleuEvent == "SPELL_SUMMON" then
                    local destGUID = cleu.destGUID
                    local _, _, _, _, _, _, _, creatureId = find(destGUID, "(%S+)-(%d+)-(%d+)-(%d+)-(%d+)-(%d+)-(%S+)")
                    creatureId = tonumber(creatureId)
                    local now = GetTime()
                    for id, v in pairs(demonData) do
                        if id == creatureId then
                            self.demonsCount[destGUID] = {
                                id = creatureId,
                                timestamp = now,
                                finish = now + v.duration
                            }
                            break
                        end
                    end
                    for k, d in pairs(self.demonsCount) do
                        if d.finish < now then
                            self.demonsCount[k] = nil
                        end
                    end
                    self.ovale:needRefresh()
                elseif cleuEvent == "SPELL_CAST_SUCCESS" then
                    local header = cleu.header
                    local spellId = header.spellId
                    if spellId == 196277 then
                        for k, d in pairs(self.demonsCount) do
                            if d.id == 55659 or d.id == 143622 then
                                self.demonsCount[k] = nil
                            end
                        end
                        self.ovale:needRefresh()
                    end
                    local aura = customAuras[spellId]
                    if aura then
                        self:addCustomAura(aura.customId, aura.stacks, aura.duration, aura.auraName)
                    end
                end
            end
        end
        self.impsSpawnedDuring = function(positionalParams, namedParams, atTime)
            local ms = positionalParams[1]
            local delay = (ms or 0) / 1000
            local impsSpawned = 0
            if self.future.next.currentCast.spellId == 105174 then
                local soulshards = self.power.current.power["soulshards"] or 0
                if soulshards >= 3 then
                    soulshards = 3
                end
                impsSpawned = impsSpawned + soulshards
            end
            local talented = self.ovaleSpellBook:getTalentPoints(23146) > 0
            if talented then
                local value = self:getRemainingDemonDuration(143622, atTime + delay)
                if value <= 0 then
                    impsSpawned = impsSpawned + 1
                end
            end
            return returnConstant(impsSpawned)
        end
        self.getDemonsCount = function(positionalParams, namedParams, atTime)
            local creatureId = positionalParams[1]
            local count = 0
            for _, d in pairs(self.demonsCount) do
                if d.finish >= atTime and d.id == creatureId then
                    count = count + 1
                end
            end
            return returnConstant(count)
        end
        self.demonDuration = function(positionalParams, namedParams, atTime)
            local creatureId = positionalParams[1]
            local value = self:getRemainingDemonDuration(creatureId, atTime)
            return returnConstant(value)
        end
        self.timeToShard = function(positionalParams, namedParams, atTime)
            local value = self:getTimeToShard(atTime)
            return returnConstant(value)
        end
        ovale:createModule("OvaleWarlock", self.handleInitialize, self.handleDisable)
    end,
    registerConditions = function(self, condition)
        condition:registerCondition("timetoshard", false, self.timeToShard)
        condition:registerCondition("demons", false, self.getDemonsCount)
        condition:registerCondition("demonduration", false, self.demonDuration)
        condition:registerCondition("impsspawnedduring", false, self.impsSpawnedDuring)
    end,
    cleanState = function(self)
    end,
    initializeState = function(self)
    end,
    resetState = function(self)
    end,
    getRemainingDemonDuration = function(self, creatureId, atTime)
        local max = 0
        for _, d in pairs(self.demonsCount) do
            if d.finish >= atTime and d.id == creatureId then
                local remaining = d.finish - atTime
                if remaining > max then
                    max = remaining
                end
            end
        end
        return max
    end,
    addCustomAura = function(self, customId, stacks, duration, buffName)
        local now = GetTime()
        local expire = now + duration
        self.ovaleAura:gainedAuraOnGUID(self.ovale.playerGUID, now, customId, self.ovale.playerGUID, "HELPFUL", false, nil, stacks, nil, duration, expire, false, buffName, nil, nil, nil)
    end,
    getTimeToShard = function(self, atTime)
        local value = 3600
        local tickTime = 2 / self.ovalePaperDoll:getHasteMultiplier("spell", atTime)
        local activeAgonies = self.ovaleAura:auraCount(980, "HARMFUL", true, nil, atTime, nil)
        if activeAgonies > 0 then
            value = ((1 / (0.184 * pow(activeAgonies, -2 / 3))) * tickTime) / activeAgonies
            if self.ovaleSpellBook:isKnownTalent(19281) then
                value = value * 0.85
            end
        end
        return value
    end,
})
